/**
 * Measure coverage and save report to file.
 * @memberof module:ap-covering/lib
 * @function measureCoverage
 * @param {string} script - JavaScript script to measure
 * @param {string[]} scriptArgs - Arguments to pass script.
 * @param {object} {object} [options] - Optional settings.
 * @param {string} [options.dir='coverage'] - Directory path to save.
 * @returns {Promise}
 */

'use strict'

const argx = require('argx')
const execcli = require('execcli')
const co = require('co')

let deprecateCallback = () => '[ape-covering] Callback is now deprecated. Use promise interface instead.'

/** @lends measureCoverage */
function measureCoverage (script, scriptArgs, options) {
  let args = argx(arguments)
  let callback = args.pop('function') || argx.noop

  script = args.shift('string')
  scriptArgs = [].concat(args.shift('string|array') || [])
  options = args.pop('object') || {}

  let spawnArgs = [ 'cover', '--dir', options.dir || 'coverage', script, '--' ].concat(scriptArgs)

  return co(function * () {
    let result = yield exports.istanbul(spawnArgs)
    if (callback) {
      deprecateCallback()
      callback(null, result)
    }
    return result
  }).catch((err) => {
    if (callback) {
      deprecateCallback()
      callback(err)
    }
    return Promise.reject(err)
  })
}

exports.istanbul = function (args) {
  let options = {
    notfound: 'try `npm install istanbul -g`'
  }
  return execcli('istanbul', args, options)
}

module.exports = measureCoverage
